@@ -71,6 +71,9 @@ public:
   KProcess save;
   KProcess load;
   KProcess stop;
+  KProcess setmark;
+  KProcess markaddp;
+  KProcess markdelp;
   KProcess stop_nat;
   KProcess stop_mangle;
   KProcess polrules;
@@ -122,7 +125,21 @@ public slots:
   void masqoffDone();
   void clearLog();
 
-   /* interface functions */
+
+  /* list mark rules functions */
+  void markRules();
+  void addmarkRule();
+  void delmarkRule();
+  void setmarkRules(KProcess *, char *, int);
+  void closemarkRules();
+  void addmarkDone();
+  void delmarkDone();
+  void markchainActivated(int);
+  void markpolicyActivated(int);
+  void markflagsActivated(int);
+  
+
+  /* interface functions */
   void startprobeInterface();
   void probeInterface(KProcess *, char *, int);
   void probeInterface2(KProcess *, char *, int); 
@@ -140,6 +157,7 @@ public slots:
   void NMap();
   void portDump(KProcess *, char *, int);
 
+
   /* list rules functions */
   void iptablesRules();
   void closeiptablesRules();
@@ -149,11 +167,13 @@ public slots:
 
   void interfaceActivated(int);
   void interfacenrActivated(int);
+  void stateActivated(int);
   void chainActivated(int);
   void policyActivated(int);
   void flagsActivated(int);
   void polchainActivated(int);
 
+
   /* chain policy options */
   void polRules();
   void closepolRules();
@@ -251,6 +271,8 @@ private:
   QLineEdit *thirdNet;
   QLineEdit *thirdNetMask;
   QLineEdit *thirdPort;
+  QLineEdit *markNumber;
+  QLineEdit *markRuleNumber;
   QLineEdit *addnatRuleNumber;
   QLineEdit *addmasqRuleNumber;
   QLineEdit *masqSPortNumber;
@@ -274,6 +296,7 @@ private:
   QLineEdit *mapNetMask;
 
   KNFWidget *polList;
+  KNFWidget *markList;
   KNFWidget *natList;
   KNFWidget *natpolList;
   KNFWidget *masqList;
@@ -292,6 +315,9 @@ private:
   QComboBox *interfacenr;
   QComboBox *chain;
   QComboBox *policy;
+  QComboBox *markchain;
+  QComboBox *markpolicy;
+  QComboBox *markflags;
   QComboBox *nat;
   QComboBox *natinterface;
   QComboBox *natinterfacenr;
@@ -300,6 +326,7 @@ private:
   QComboBox *masqinterfacenr;
   QComboBox *masqprotocol;
   QComboBox *flags;
+  QComboBox *state;
   QComboBox *polchain;
   QComboBox *polchainpolicy;
   QComboBox *natchain;
@@ -323,7 +350,7 @@ private:
   QCheckBox *manglelog2;
   QCheckBox *manglelog3;
 
-  /* the menubar, toolbar and statusbar */
+  /* the menubar, toolbar statusbar and marktoolbar*/
   KMenuBar *menubar;
   KToolBar *toolbar;
   KStatusBar *statusbar;
@@ -346,8 +373,13 @@ private:
   char *interfaceChoicemasq;
   char *ruleaction;
   char *chainChoice;
+  char *stateChoice;
   char *policyChoice;
   char *flagChoice;
+  char *markruleaction;
+  char *markchainChoice;
+  char *markpolicyChoice;
+  char *markflagChoice;
   char *natprotocolChoice;
   char *polchainChoice;
   char *chainpolicyChoice;
@@ -371,7 +403,7 @@ private:
   char *mangleSyn;
   char *nmapopt;
   char *nmapopt_icmp;
-  char* logLevel;
+  char *logLevel;
   char *log1;
   char *log2;
   char *log3;
