@@ -944,6 +944,30 @@ void knetfilter::interfacenrActivated(int index) {
 };
 
 
+void knetfilter::stateActivated(int index) {
+  switch(index) {
+  default:
+    stateChoice = "nostate";
+  case 0:
+    stateChoice = "nostate";
+    break;
+  case 1:
+    stateChoice = "INVALID";
+    break;
+  case 2:
+    stateChoice = "ESTABLISHED";
+    break;
+  case 3:
+    stateChoice = "NEW";
+    break;
+  case 4:
+    stateChoice = "RELATED";
+    break;
+  };
+
+};
+
+
 void knetfilter::chainActivated(int index) {
   switch(index) {
   default:
