@@ -92,22 +92,46 @@ void knetfilter::addRule() {
       QMessageBox::critical(0, "Error",
       			"Not a valid netmask address.\n");
     else {
-          if (dport.length()==0)
-	{
-	  addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-	  addp2 << "iptables" << ruleaction << chainChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-	}
+      if (strcmp("nostate",stateChoice)==0)
+      {
+	if (dport.length()==0)
+	  {
+	    addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	    addp2 << "iptables" << ruleaction << chainChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	  }
+	else
+	  {
+	    if (sport.length()==0)
+	      {
+		addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	      }
+	    else
+	      {
+		addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	      }
+	  }
+      }
       else
 	{
-	  if (sport.length()==0)
+	  if (dport.length()==0)
 	    {
-	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-	      addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	      addp2 << "iptables" << ruleaction << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 	    }
 	  else
 	    {
-	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-	      addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	      if (sport.length()==0)
+		{
+		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		}
+	      else
+		{
+		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		}
 	    }
 	}
     };
@@ -191,23 +215,44 @@ void knetfilter::addRule_tcp() {
       QMessageBox::critical(0, "Error",
       			"Not a valid netmask address.\n");
     else {
-      if (dport.length()==0)
+      if (strcmp("nostate",stateChoice)==0)
 	{
-	  addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
-	}
-      else
-	{
-	  if (sport.length()==0)	
+	  if (dport.length()==0)
 	    {
-	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 	    }
 	  else
 	    {
-	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	      if (sport.length()==0)	
+		{
+		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		}
+	      else
+		{
+		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		}
+	    }      
+	}
+      else
+	{
+	  if (dport.length()==0)
+	    {
+	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 	    }
-	}      
+	  else
+	    {
+	      if (sport.length()==0)	
+		{
+		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		}
+	      else
+		{
+		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		}
+	    }      	  
+	}	
     };
-
+      
     bool success=addp.start();
     if (success)
       statusbar->message(i18n("Running..."), 2000);
@@ -285,21 +330,42 @@ void knetfilter::addRule_udp() {
       QMessageBox::critical(0, "Error",
       			"Not a valid netmask address.\n");
     else {
-      if (dport.length()==0)
+      if (strcmp("nostate",stateChoice)==0)
 	{
-	  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
-	}
-      else
-	{
-	  if (sport.length()==0)	
+	  if (dport.length()==0)
 	    {
-	      addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+	      addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 	    }
 	  else
 	    {
-	      addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	      if (sport.length()==0)	
+		{
+		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		}
+	      else
+		{
+		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		}
 	    }
-	}      
+	}
+      else
+	{
+	  if (dport.length()==0)
+	    {
+	      addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+	    }
+	  else
+	    {
+	      if (sport.length()==0)	
+		{
+		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		}
+	      else
+		{
+		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		}
+	    }
+	}
     };
 
     bool success2=addp2.start();
@@ -379,7 +445,14 @@ void knetfilter::addRule_icmp() {
       QMessageBox::critical(0, "Error",
       			"Not a valid netmask address.\n");
     else {
-      addp3 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "icmp" << "--icmp-type" << type << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+      if (strcmp("nostate",stateChoice)==0)
+	{
+	  addp3 << "iptables" << ruleaction << chainChoice << rulepos << "-p" << "icmp" << "--icmp-type" << type << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	}
+      else
+	{
+	  addp3 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "icmp" << "--icmp-type" << type << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	}
     };
 
     bool success=addp3.start();
@@ -458,22 +531,46 @@ void knetfilter::delRule() {
 	  QMessageBox::critical(0, "Error",
 				"Not a valid netmask address.\n");
 	else {
-	  if (dport.length()==0)
+	  if (strcmp("nostate",stateChoice)==0)
 	    {
-	      delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-	      delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-	    }
-	  else
-	    {
-	      if (sport.length()==0)
+	      if (dport.length()==0)
 		{
-		  delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-		  delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 		}
 	      else
 		{
-		  delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-		  delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  if (sport.length()==0)
+		    {
+		      delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		      delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		    }
+		  else
+		    {
+		      delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		      delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		    }
+		}
+	    }
+	  else
+	    {
+	      if (dport.length()==0)
+		{
+		  delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		}
+	      else
+		{
+		  if (sport.length()==0)
+		    {
+		      delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		      delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		    }
+		  else
+		    {
+		      delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		      delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		    }
 		}
 	    }
 	}
@@ -556,23 +653,44 @@ void knetfilter::delRule_tcp() {
 	  QMessageBox::critical(0, "Error",
 				"Not a valid netmask address.\n");
 	else {
-	  if (dport.length()==0)
+	  if (strcmp("nostate",stateChoice)==0)
 	    {
-	      delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
-	    }
-	  else
-	    {
-	      if (sport.length()==0)	
+	      if (dport.length()==0)
 		{
-		  delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		  delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 		}
 	      else
 		{
-		  delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  if (sport.length()==0)	
+		    {
+		      delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		    }
+		  else
+		    {
+		      delp << "iptables" << "-D" << chainChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		    }
+		}      
+	    }
+	  else
+	    {
+	      if (dport.length()==0)
+		{
+		  delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 		}
-	    }      
+	      else
+		{
+		  if (sport.length()==0)	
+		    {
+		      delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		    }
+		  else
+		    {
+		      delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		    }
+		}      
+	    }
 	}
-    };
+      };
 
     bool success=delp.start();
 
@@ -650,21 +768,42 @@ void knetfilter::delRule_udp() {
 	  QMessageBox::critical(0, "Error",
 				"Not a valid netmask address.\n");
 	else {
-	  if (dport.length()==0)
+	  if (strcmp("nostate",stateChoice)==0)
 	    {
-	      delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
-	    }
-	  else
-	    {
-	      if (sport.length()==0)	
+	      if (dport.length()==0)
 		{
-		  delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		  delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 		}
 	      else
 		{
-		  delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  if (sport.length()==0)	
+		    {
+		      delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		    }
+		  else
+		    {
+		      delp2 << "iptables" << "-D" << chainChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		    }
 		}
-	    }      
+	    }
+	  else
+	    {
+	      if (dport.length()==0)
+		{
+		  delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		}
+	      else
+		{
+		  if (sport.length()==0)	
+		    {
+		      delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		    }
+		  else
+		    {
+		      delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		    }
+		}
+	    }
 	}
     };
 
@@ -743,7 +882,14 @@ void knetfilter::delRule_icmp() {
 	  QMessageBox::critical(0, "Error",
 				"Not a valid netmask address.\n");
 	else {
-	  delp3 << "iptables" << "-D" << chainChoice << "-p" << "icmp" << "--icmp-type" << type << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	  if (strcmp("nostate",stateChoice)==0)
+	    {
+	      delp3 << "iptables" << "-D" << chainChoice << "-p" << "icmp" << "--icmp-type" << type << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	    }
+	  else
+	    {
+	      delp3 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "icmp" << "--icmp-type" << type << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	    }
 	}
       };
 	
@@ -970,7 +1116,7 @@ void knetfilter::addDone_tcp() {
     addDestinationPort->clear();
     addRuleNumber->clear();
   } else
-    statusbar->message(i18n("iptables died"), 2000);
+  statusbar->message(i18n("iptables died"), 2000);
   addSourcePort->setFocus();
   addDestinationPort->setFocus();
   addRuleNumber->setFocus();
