@@ -1,8 +1,8 @@
   /*
 
-  knat v 2.0.3
+  knat v 2.0.5
 
-  Copyright (C) 2000 Luigi Genoni
+  Copyright (C) 2001 Luigi Genoni
   		     venom@DarkStar.sns.it
   */
   
@@ -38,8 +38,12 @@ knetfilter::knetfilter( QWidget *, const char *name )
     masqinterfacenrChoice = 0;
     masqinterfacenrChoicedmp = "0";
     chainChoice = "INPUT";
+    stateChoice = "nostate";
     policyChoice = "DROP";
     flagChoice = "-v";
+    markchainChoice = "INPUT";
+    markpolicyChoice = "DROP";
+    markflagChoice = "-v";
     polchainChoice = "INPUT";
     chainpolicyChoice = "ACCEPT";
     natchainChoice= "PREROUTING";
@@ -78,15 +82,16 @@ knetfilter::knetfilter( QWidget *, const char *name )
 	 * Initialize pointers -- this is necassary for
 	 * *List == NULL test to work later.
 	 */
-	polList = NULL;
-	natList = NULL;
-	natpolList = NULL;
-	masqList = NULL;
-	mangleList = NULL;
-	manglepolList = NULL;
-	dumpList = NULL;
-	mapList = NULL;
-	masqWin = NULL;
+    markList = NULL;
+    polList = NULL;
+    natList = NULL;
+    natpolList = NULL;
+    masqList = NULL;
+    mangleList = NULL;
+    manglepolList = NULL;
+    dumpList = NULL;
+    mapList = NULL;
+    masqWin = NULL;
 };
 
 
@@ -151,24 +156,27 @@ void knetfilter::setupMenuBar() {
   tables->insertItem(i18n("Del rule to icmp"), 
   				this, SLOT(delRule_icmp()));
   tables->insertSeparator();
+  tables->insertItem(i18n("M&ark based rules"),
+		                this, SLOT(markRules()));
+  tables->insertSeparator();
   tables->insertItem(i18n("Flush iptables &rules"), 
 		                this, SLOT(stopAll()), CTRL+Key_R);
 
   
   useMasq = nat_tables->insertItem(i18n("Masquerade"), 
 				this, SLOT(masqRules()));
-   nat_tables->insertItem(i18n("Redirections"),
+  nat_tables->insertItem(i18n("Redirections"),
   			        this, SLOT(natRules()));
-   nat_tables->insertItem(i18n("Define Nat Chain policy"),
+  nat_tables->insertItem(i18n("Define Nat Chain policy"),
 			        this, SLOT(natpolRules()));
-   nat_tables->insertItem(i18n("Flush Nat Rules"),
+  nat_tables->insertItem(i18n("Flush Nat Rules"),
 			        this, SLOT(stopNAT()), CTRL+Key_N);
-   mangle->insertSeparator();
-   mangle->insertItem(i18n("Mangle"),
+  mangle->insertSeparator();
+  mangle->insertItem(i18n("Mangle"),
                 		this, SLOT(mangleRules())); 
-   mangle->insertItem(i18n("Define Mangle Chain policy"),
+  mangle->insertItem(i18n("Define Mangle Chain policy"),
                 		this, SLOT(manglepolRules())); 
-   mangle->insertItem(i18n("Flush Mangle Rules"),
+  mangle->insertItem(i18n("Flush Mangle Rules"),
                 		this, SLOT(stopMangle()), CTRL+Key_M); 
 
   options->insertSeparator();
@@ -235,6 +243,10 @@ void knetfilter::setupToolBar() {
 			this,SLOT(addRule_icmp()),TRUE,i18n("Add rule to icmp")); 
   toolbar->insertButton("stop_icmp.xpm",del,SIGNAL(clicked()),
 			this,SLOT(delRule_icmp()),TRUE,i18n("Delete rule to icmp"));
+  toolbar->insertSeparator();
+  toolbar->insertButton("mark.xpm",del,SIGNAL(clicked()),
+			this,SLOT(markRules()),TRUE,i18n("Set Up Mark based rules"));
+  toolbar->insertSeparator();
   toolbar->insertButton("probe.xpm",del,SIGNAL(clicked()),
 			this,SLOT(startprobeInterface()),TRUE,i18n("Probe Network Interface"));
   toolbar->setBarPos( KToolBar::Top );
